#!/usr/bin/perl
#
# log when a user "logs into the network"
# and generate a custom logon script
#
#       the software is GNU licensing dependent.
# Copyright (C) 2000, schoolLAN (tm), Stichting NLnet
#
# This program is free software; you van redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTLY; without even the implied warranty of
# MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: makelogonscript.pl,v 4.30 2004/09/15 19:40:57 teus Exp $

$verbose=" >nul";

sub usage {
	print STDERR "Usage: [-<key-flag_letter><string>] [args ....]
where:
	-V[verbose]	logon bat file will not be silent; default: silent.
	-Uuser		user, default: via UNIX \"who am I\" command.
	-Cclient.domain	Windows client DNS name; default: UNKNOWN.
	-Bclient	Windows client Bios Network name; default UNKNOWN.
	-Ssamba-server	Netbios name of the server; default: taken from /etc/samba/smb.conf.
	-Aarchitecture	Windows OS type: Win95, WinNT, Win2K, WinXP; default: ERROR.
	-IclientIP	Windows Client IP number; default: 192.168.168.254.
	-print		Print the Windows Registry Database
	-Pprotocol	Windows smb protocol type: default: WinNT.
	-Help		This message.
	-Hhome		UNIX home directory user: default none.
	-Nprofiles_dir	Netlogon profiles subdirectory in user home; default none.

	-Ffile_name	File_name for policy collection for registry settings.
	-Ooutput_file	Output File name for Windows Registry settings (ascii).
	-HKEY_{LOCAL_MACHINE|USERS}  type of hive key (more as one is possible)
	-Hive=*         all type of hive key (more as one -Hive=??? may be defined)
			-Hive=HKU for HKEY_USERS is also permitted, HKLM etc.
	-Type=STRING	use enabled values from database, enabled by matching the string
			-Type=init or -Type=logon or -Type=install

The arguments are well ordered, where some settings may be omitted if
already given via the flagged arguments.
The ordered arguments are:
user_name NBclient_name smbserver_name home_dir_user profiles_sub_dir WinXX protocol IP_client

The command with only the Win?? architecture (e.g. minimal with -AWinXP) defined acts as a filter:
	stdin: lines with key[=value] e.g. DontDisplayLastUserName=ON or Comment=BlaBla
	stdout: registry settings with REGEDIT4 header.

";
	exit 0;
}

sub error {
	my ($string) = @_;
	if ( defined $netlogon && defined $home && defined $user && defined $client ) {
            system "logger -t SMBclient -p local3.warning \"Reg setting error on $user for $client: $string)\"";
	} else {
	    print STDERR $string . "\n" ;
	}
}

############### REGISTRY HANDLING and generation of a .reg file ########################
# registry settings which might be set if enabled
# values are either "string" or dword:00000000 or hex:0a,00,00,00
# win says for which windos OS type: scope till next is defined
# enabled will say yes use the value anyway when enable flag matches.
# hive scope: till next is defined.
# discrimination on keys is now rather simple: [hive_key_part\]key
# so we can discriminate for AccessTop\Installed=0 and Calc\Installed=1
# how to:
# how to delete a value: via key_name=-
# how to delete a hive: via -[HKLM/.../]
# 	in >Win95 arch make sure the hive has no branches still pending...

$OFF="dword:00000000" ;
$ON="dword:00000001" ;
@WindowsReg = (
    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\Class\\NetTrans\\0001" ,
        win => "Win95" ,
        key => "IPAddress", value => "\"%IPCLIENT%\"" , },
      { key => "IPMask", value => "\"%MASK%\"" , },
      { key => "DefaultGateway", value => "\"%SERVERIP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\Class\\NetTrans\\0002" ,
        key => "IPAddress", value => "\"%IPCLIENT%\"" , },
      { key => "IPMask", value => "\"%MASK%\"" , },
      { key => "DefaultGateway", value => "\"%SERVERIP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\Class\\NetTrans\\0003" ,
        key => "IPAddress", value => "\"%IPCLIENT%\"" , },
      { key => "IPMask", value => "\"%MASK%\"" , },
      { key => "DefaultGateway", value => "\"%SERVERIP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\Class\\NetTrans\\0004" ,
        key => "IPAddress", value => "\"%IPCLIENT%\"" , },
      { key => "IPMask", value => "\"%MASK%\"" , },
      { key => "DefaultGateway", value => "\"%SERVERIP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\VxD\\VNETSUP" ,
        key => "ComputerName", value => "\"%CLIENT%\"" , },
      { key => "Workgroup", value => "\"%WORKGROUP%\"" , },
      { key => "Comment", value => "\"Loaded by %USER% at %DATE%\"" , },
      { key => "FileSharing", value => "\"Yes\"" , },
      { key => "PrintSharing", value => "\"No\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\VxD\\MSTCP" ,
        key => "EnableDNS", value => "\"1\"" , },
      { key => "HostName", value => "\"%DNSCLIENT%\"" , },
      { key => "Domain", value => "\"%DOMAINNAME%\"" , },
      { key => "SearchList", value => "\"%DOMAINNAME%\"" , },
      { key => "NameServer", value => "\"%SERVERIP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\MSNP32\\NetworkProvider" ,
        key => "AuthenticatingAgent", value => "\"%WORKGROUP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ComputerName\\ComputerName" ,
        key => "ComputerName", value => "\"%CLIENT%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\Netlogon\\Parameters" ,
        key => "RegisterDnsARecords", value => $OFF , } ,

    { hive => "HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters" ,
        key => "DisableReverseAddressRegistrations", value => $OFF , } ,

    { hive => "HKEY_LOCAL_MACHINE\\Network\\Logon" ,
        key => "PrimaryProvider", value => "\"Microsoft Network\"" , },
      { key => "PolicyHandler", value => "\"GROUPPOL.DLL,ProcessPolicies\"" , },
      { key => "logonvalidated", value => "hex:01,00,00,00" , },
      { key => "DontShowLastUser", value => $ON , },
      { key => "UserProfiles", value => $ON , },
      { key => "LMLogon" , value => $ON , } ,
      { key => "NoDomainPwdCaching" , value => $ON , } ,

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion" ,
        key => "RegisteredOrganization" , value => "\"%USERNAME%\"" , },
      { key => "RegisteredOwner" , value => "\"%USERNAME%\"" , },
      { key => "ProductId" , value => "\"UNKNOWN\"", },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Network" ,
        key => "HideSharePwds", value => $ON , },
      { key => "DisablePwdCaching", value => $ON , },
      { key => "MinPwdLen", value => "dword:00000005" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services\\MSNP32\\NetworkProvider" ,
        key => "AuthenticatingAgent", value => "\"%WORKGROUP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\TimeZoneInformation" ,
        key => "DisableAutoDayLightTimeSet", value => $ON , },

    { hive => "HKEY_LOCAL_MACHINE\\Security\\Provider" ,
        key => "Platform_Type", value => "hex:00,00,00,00" , },
      { key => "Container", value => "\"%WORKGROUP%\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Setup" ,
        key => "SourcePath", value => "\"I:\\Win98SEcab\\\"" , },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Network\\LanMan\\ROOT\$" ,
        key => "Remark", value => "\"schoolLAN $Revision: 2.0 %DATE% by %USER%\"" , },
      { key => "Flags", value => "dword:00000192" , },
      { key => "Type", value => "dword:00000000" , },
      { key => "Path", value => "\"C:\\\"" , },
      { key => "Parm2enc", value => "hex:" , },
      { key => "Parm1enc", value => "hex:" , },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileList\\%USER%" ,
        win => "Win95" ,
        enabled => "logon" , 
        key => "ProfileImagePath" ,
        value => "\"C:\\Windows\\Profiles\\%USER%\"" , } ,
    { hive => "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders",
        win => "Win95|Win2K|WinNT|WinXP" ,
        key => "AppData" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Application Data\"" , } ,
      { key => "Cache" ,
        win => "Win95" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Temporary Internet Files\"" , } ,
      { key => "Favorites" ,
        win => "Win95|Win2K|WinNT|WinXP" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Favorieten\"" , } ,
      { key => "Cookies" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Cookies\"" , } ,
      { key => "History" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Geschiedenis\"" , } ,
      { key => "Personal" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Mijn Documenten\"" , } ,

    { hive => "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
        win => "Win95|Win2K|WinNT|WinXP" ,
        key => "AppData" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Application Data\"" , } ,
      { key => "Favorites" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Favorieten\"" , } ,
      { key => "Cookies" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Cookies\"" , } ,
      { key => "History" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Geschiedenis\"" , } ,
      { key => "Personal" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Mijn Documenten\"" , } ,
      { key => "Desktop" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Desktop\"" , } ,
      { key => "PrintHood" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\PrintHood\"" , } ,
      { key => "Start Menu" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Start Menu\"" , } ,
      { key => "Programs" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Start Menu\\Programmas\"" , } ,
      { key => "Startup" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Start Menu\\Startup\"" , } ,
      { key => "Recent" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Recent\"" , } ,
      { key => "SendTo" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\SendTo\"" , } ,
      { key => "Templates" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Templates\"" , } ,
      { key => "Cache" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Temporary Internet Files\"" , win => "Win95" , } ,
      { key => "Cache" ,
        win => "Win2K|WinNT|WinXP" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Local Settings\\Temporary Internet Files\"" , } ,
      { key => "Local AppData" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Local Settings\\Geschiedenis\"" , } ,
      { key => "Local Settings" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Local Settings\\Application Data\"" , } ,
      { key => "Cookies" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\Cookies\"" , } ,
      { key => "My Pictures" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Mijn Documenten\\Mijn Plaatjes\"" , } ,
      { key => "PrintHood" ,
        value => "\"\\\\%LOGONSERVER%\\%USERNAME%\\Profiles\\PrintHood\"" , } ,
  
    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer" ,
        key => "NoRun" , value => $ON , win => "Win95|WinNT|Win2K|WinXP" , } ,
      { key => "NoSetFolders" , value => $ON , },
      { key => "NoSetTaskbar" , value => $ON , },
      { key => "NoFind" , value => $ON , },
      { key => "NoNetHood" , value => $ON , },
      { key => "NoDesktop" , value => $ON , },
      { key => "RestrictRun" , value => $ON , },
      { key => "NoStartMenuSubFolders" , value => $ON , },
      { key => "EnforceShellExtensionSecurity" , value => $ON , },
      { key => "NoFileMenu" , value => $ON , },
      { key => "NoCommonGroups" , value => $ON , },
      { key => "NoTrayContextMenu" , value => $ON , },
      { key => "NoViewContextMenu" , value => $ON , },
      { key => "NoNetConnectDisconnect" , value => $ON , },
      { key => "LinkResolveIgnoreLinkInfo" , value => $ON , },
      { key => "NoSaveSettings" , value => $ON , },
        # explorer displays correct filename capitalisation
      { key => "DontPrettyPath" , value => $ON , },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer\\MenuOrder" ,
        # Start Menu alphabetically
        win => "Win95|WinNT|Win2K|WinXP" ,
        key => "MenuOrder" , value => "-" , } ,

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion" ,
        key => "RegisteredOrganization" , value => "\"UNKNOWN\"" , },
      { key => "RegisteredOwner" , value => "\"UNKNOWN\"" , },
      { key => "ProductId" , value => "\"UNKNOWN\"", },

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon" ,
        win => "WinNT|Win2K|WinXP" ,
        key => "RunLogonScriptSync" , value => $ON , } ,
      { key => "LegalNoticCaption" , vlaue => "schoolLAN" , } ,
      { key => "LegalNoticText" , value => "\"under GPL license model\"" , } ,
      { key => "ShutdownWithoutLogon" , value => $ON , } ,
      { key => "DontDisplayLastUserName" , value => $ON , } ,
      { key => "DeleteRoamingCache" , value => $OFF , } ,
      { key => "ShowLinkDetectEnabled" , value => $ON , } ,

    { hive => "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer\\Tips" ,
        win => "WinNT|Win2K|WinXP" ,
        key => "Show" , value => $ON , } ,
    { hive => "HKEY_CURRENT_USER\\Control Panel\\Desktop" ,
        win => "Win95|Win2K|WinNT|WinXP" ,
        # use true color icons
        key => "UserPreferencesMask" , value => "hex:9c,00,00,80" , } ,
  
    # next will be manipulation on Win95 USER.DAT registry
    { hive => "HKEY_USERS\\Software\\Microsoft\\MediaPlayer\\Setup\\CreatedLinks" ,
        win => "Win95" ,
        key => "@" , value => "\"Windows Media Player \"" , },
      { key => "Shortcut0", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\\\\Bureau-accessoires\\\\Entertainment\\\\Windows Media Player .lnk\"" , enabled => "init", },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders" ,
        win => "Win95" ,
        key => "AppData", value => "\"H:\\\\Profiles\\\\Application Data\"" , enabled => "init", },
      { key => "Cache", value => "\"H:\\\\Profiles\\\\Local Settings\\\\Temporary Internet Files\"" , enabled => "waitinit", },
      { key => "Cookies", value => "\"H:\\\\Profiles\\\\Cookies\"" , enabled => "init", },
      { key => "Desktop", value => "\"H:\\\\Profiles\\\\Desktop\"" , enabled => "init", },
      { key => "Favorites", value => "\"H:\\\\Favorieten\"" , enabled => "init", },
      { key => "Fonts", value => "\"C:\\\\WINDOWS\\\\Fonts\"" , },
      { key => "History", value => "\"H:\\\\Profiles\\\\Geschiedenis\"" , enabled => "init", },
      { key => "NetHood", value => "\"H:\\\\Profiles\\\\NetHood\"" , enabled => "init", },
      { key => "Personal", value => "\"H:\\\\Mijn documenten\"" , enabled => "init", },
      { key => "Programs", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\"" , enabled => "init", },
      { key => "Recent", value => "\"H:\\\\Profiles\\\\Recent\"" , enabled => "init", },
      { key => "SendTo", value => "\"H:\\\\Profiles\\\\SendTo\"" , enabled => "init", },
      { key => "Start Menu", value => "\"H:\\\\Profiles\\\\Start Menu\"" , enabled => "init", },
      { key => "Startup", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\\\\Opstarten\"" , enabled => "init", },
      { key => "Templates", value => "\"C:\\\\WINDOWS\\\\ShellNew\"" , },
	# next are not in std USER.DAT file
      { key => "PrintHood", value => "\"H:\\\\Profiles\\\\PrintHood\"" , enabled => "waitinit", },
      { key => "Local AppData", value => "\"H:\\\\Profiles\\\\Application Data\"" , enabled => "waitinit", },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders" ,
        win => "Win95" ,
        key => "AppData", value => "\"H:\\\\Profiles\\\\Application Data\"" , enabled => "init", },
      { key => "Cache", value => "hex(2):43,3a,5c,57,49,4e,44,4f,57,53,5c,54,65,6d,70,6f,72,61,72,79,20,49,6e,74,65,72,6e,65,74,20,46,69,6c,65,73,00" , },
      { key => "Cookies", value => "\"H:\\\\Profiles\\\\Cookies\"" , enabled => "init",},
      { key => "Favorites", value => "\"H:\\\\Favorieten\"" , enabled => "init", },
      { key => "History", value => "\"H:\\\\Profiles\\\\Geschiedenis\"" , enabled => "init", },
      { key => "Personal", value => "\"H:\\\\Mijn documenten\"" , enabled => "init", },
	# next are not in std USER.DAT file
      { key => "Desktop", value => "\"H:\\\\Profiles\\\\Desktop\"" , enabled => "init", },
      { key => "NetHood", value => "\"H:\\\\Profiles\\\\NetHood\"" , enabled => "init", },
      { key => "Programs", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\"" , enabled => "init", },
      { key => "Recent", value => "\"H:\\\\Profiles\\\\Recent\"" , enabled => "init", },
      { key => "Start Menu", value => "\"H:\\\\Profiles\\\\Start Menu\"" , enabled => "init", },
      { key => "Startup", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\\\\Opstarten\"" , enabled => "init", },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation" ,
	win => "Win95" ,
#	key => "ProfileDirectory" , value => "\"C:\\\\WINDOWS\\Profiles\\%USER%\"", enabled => "init", } ,
	key => "ProfileDirectory" , value => "\"H:\\\\Profiles\"", enabled => "init", } ,
    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Cookies" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Cookies\"" , enabled => "waitinit", },
      { key => "DefaultDir", value => "\"H:\\\\Profiles\\\\Cookies\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"Cookies\"" , },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Cookies\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Desktop" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Desktop\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Profiles\\\\Desktop\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Desktop\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Favorites" ,
        key => "CentralFile", value => "\"H:\\\\Favorieten\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Favorieten\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Favorites\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\History" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Geschiedenis\"" , enabled => "waitinit", },
      { key => "DefaultDir", value => "\"H:\\\\Profiles\\\\Geschiedenis\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"Geschiedenis\"" , },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Personal" ,
        key => "CentralFile", value => "\"H:\\\\Mijn documenten\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Mijn documenten\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Personal\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Protected Storage System Provider\\%USER%" ,
        key => "Migrate" , value => "dword:00000003" , enabled => "waitinit" , } ,

    { hive => "HKEY_USERS\\Software\\Microsoft\\Protected Storage System Provider\\%USER%\\Data 2" , enabled => "waitinit",  },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Protected Storage System Provider\\%USER%\\Data 2\\Windows" ,
       key => "Value", value => "hex:01,00,00,00,1c,00,00,00,02,00,00,00,a7,43,88,2b,4d,c7,98,48,21,df,3c,97,9c,98,52,a8,c7,d8,f7,e1,13,05,78,0c,10,00,00,00,83,11,b7,9e,35,f9,5f,08,24,54,93,86,d9,22,62,6e,14,00,00,00,77,56,bc,11,81,a8,0e,00,8d,fc,f6,d4,ad,cd,c5,da,f2,f4,28,03" , enabled => "waitinit", } ,

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Recent" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Recent\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Profiles\\\\Recent\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.*\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Recent\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation\\Start Menu" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Start Menu\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Profiles\\\\Start Menu\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.lnk,*.pif,*.url\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Start Menu\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\SecondaryProfileReconciliation\\Programs" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.lnk,*.pif,*.url\"" , },
      { key => "ParentKey", value => "\"Start Menu\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Programs\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\SecondaryProfileReconciliation\\Startup" ,
        key => "CentralFile", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\\\\Opstarten\"" , enabled => "waitinit", },
      { key => "LocalFile", value => "\"H:\\\\Profiles\\\\Start Menu\\\\Programma\\'s\\\\Opstarten\"" , enabled => "waitinit", },
      { key => "MustBeRelative", value => $ON , },
      { key => "Name", value => "\"*.lnk,*.pif,*.url\"" , },
      { key => "ParentKey", value => "\"Start Menu\"" , },
      { key => "RegKey", value => "\"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Explorer\\\\User Shell Folders\"" , },
      { key => "RegValue", value => "\"Startup\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Windows\\CurrentVersion\\ProfileReconciliation" ,
        key => "ProfileDirectory", value => "\"C:\\\\Windows\\\\Profiles\\\\%USER%\"" , },

    { hive => "HKEY_USERS\\Software\\Microsoft\\Active Setup\\Installed Components\\{44BBA840-CC51-11CF-AAFA-00AA00B6015C}" ,
        key => "Locale", value => "\"NL\"" , },
      { key => "Username", value => "\"%USER%\"" , enabled => "init", },
      { key => "Version", value => "\"5,00,2615,200\"" , },

) ;
############# end of Windows Reg Database definition #############

############# some routines to handle the databse
# substitute some patterns
# we handle them and inits them only when needed
# IP MASK SERVERIP WORKGROUP USER DATE DOMAINNAME SERVER ARCH PROT
# some are hard to guess as PROT (smb protocol) and ARCH (type of WinXX)
# and DNSCLIENT and IP (IP address of client).
sub substitute {
        my $me;
	($me) = @_; $_ = $me;
	if( /\%USER\%/ ) {
	    if ( ! defined $user  || $user eq "") {
		use Shell qw(who); my $who;
		$who = who("am","I"); chop $who; $user = $who;
		$user =~ s/.*!//; $user=~ s/\W+.*//;
                if( ! defined $domainname || $domainname eq "" ) {
		    $domainname = $who; $domainname =~ s/\W*(\w+)!.*/$1/;
		    $domainname =~ s/.*\.//;
                }
		if ( ! defined $dnsclient || $dnsclient eq "" ) {
		    $dnsclient = $who; $dnsclient =~ s/\W*(\w+)!.*/$1/;
		}
	    }
	    s/\%USER\%/$user/g;
	}
        if( /\%ARCH\%/ ) {
	    if( ! defined $arch || $arch eq "" ) {
		# this cannot happen
		error "cannot find Windows arch type for client \"$client\"";
		$arch="WinXP";
	    }
	    s/\%ARCH\%/$arch/g;
	}
        if( /\%CLIENT\%/ ) {
	    # Netbios name of the client
            if( ! defined $client || $client eq "" ) {
		# UNKNOWN client name
		error "cannot find name for client \"$dnsclient\"";
                $client="UNKNOWN";
            }
            s/\%CLIENT\%/$client/g;
        }
        if( /\%DNSCLIENT\%/ ) {
            if( ! defined $dnsclient || $dnsclient eq "" ) {
		if ( defined $client && $client ne "" ) {
		    use Shell qw(dnsdomainname);
		    $domainname = dnsdomainname ; chop $domainname;
		    $dnsclient = $client . "." . $domainname;
		}
		else {
		    # UNKNOWN client name
		    error "cannot find name for client \"$client\"";
                    $dnsclient="UNKNOWN";
		}
            }
            s/\%DNSCLIENT\%/$dnsclient/g;
        }
        if( /\%IPCLIENT\%/ ) {
	    # IP client
	    if( ! defined $ipclient || $ipclient eq "" ) {
		if( defined $client && $client ne "" ) {
		    use Shell qw(nmblookup);
		    $_ = nmblookup("$client"); chop;
		    s/querying .*\n//; s/ .*//;
		    $ipclient =$_ ;
		}
	    }
	    if( ! defined $ipclient || $ipclient eq "" ) {
		# UNKNOWN
		error "cannot find ip address for client \"$client\"";
		$ipclient="192.168.168.254";
	    }
	    s/\%IPCLIENT\%/$ipclient/g;
	}
        if( /\%PROTOCOL\%/ ) {
	    if( ! defined $protocol || $protocol eq "" ) {
		# UNKNOWN
		error "cannot find Windows smb protocol type for client \"$client\"";
		$protocol="NT1";
	    }
	    s/\%PROTOCOL\%/$protocol/g;
	}
        if(  /\%DATE\%/ ) {
            if( ! defined $datestrg || $datestrg eq "" ) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
                $datestrg = sprintf("%02.2d/%02.2d/%04.4d %02.2dh%02.2d", $mday,$mon,$year+1900,$hour,$min);
            }
            s/\%DATE\%/$datestrg/g ;
        }
        # next could be better to collect from some config
        if( /\%DOMAINNAME\%/ ) {
	    if( ! defined $domainname || $domainname eq "" ) {
		$domainname = "" ;
		my $input ; my $save=$_;
		open $input, "</etc/sysconfig/network";
		while (<$input>){
		    if( /^\s*HOSTNAME=/ ) { chop;
			s/.*HOSTNAME=\W*\w+\.([\w\.]+)/$1/ ; $domainname=$_; last;
                    }
		}
		close $input; $_=$save;
	    }
	    if( $domainname eq "" ) {
		$domainname = "schoollan2.schoolbus.nl";
		error "Cannot find domainname for client \"$client\"";
	    }
            s/\%DOMAINNAME\%/$domainname/g;
        }
        if( /\%MASK\%/ ) {
	    if( ! defined $mask || $mask eq "" ) {
		$mask = "255.255.255.128" ;
		my $input ; my $save=$_;
		open $input, "</etc/sysconfig/networkscripts/ifcfg-eth0";
		while (<$input>){ chop;
		    if( /^\s*NETMASK=/ ) {
			s/.*NETMASK=\W*([\d\.]+)[^\d]*/$1/ ; $mask=$_; last;
                    }
		    if( /^\s*IPADDR=/ ) {
			s/.*IPADDR=\W*([0-9\.]+)\W*/$1/ ;
			$serverip=$_ if ! defined $serverip;
                    }
		}
		close $input; $_=$save;
	    }
            s/\%MASK\%/$mask/g;
        }
        if( /\%SERVERIP\%/ ) {
	    if( ! defined $serverip || $serverip eq "" ) {
		$serverip = "192.168.168.2" ;
		my $input ; my $save=$_;
		open $input, "</etc/sysconfig/networkscripts/ifcfg-eth0";
		while (<$input>){ chop;
		    if( /^\s*IPADDR=/ ) {
			s/.*IPADDR=\W*([0-9\.]+)\W*/$1/ ; $serverip=$_; last;
                    }
		    if( /^\s*NETMASK=/ ) {
			s/.*NETMASK=\W*([\d\.]+)[^\d]*/$1/ ;
			$mask=$_ if ! defined $mask;
                    }
		}
		close $input; $_=$save;
	    }
            s/\%SERVERIP\%/$serverip/g;
        }
        if( /\$SMBSERVER\%/ ) {
	    if( ! defined $smbserver || $smbserver eq "" ) {
		$smbserver = "speedo" ;
		my $input, $save=$_;
                open $input, "</etc/samba/smb.conf"  ;
		while(<$input>) { chop;
		    if( /^\s+netbios\s+name\s*=/ ) {
			s/.*=\W*(\w+)\W*/$1/; $smbserver = $_; last ;
		    }
		    if( /^\s\s*workgroup\s*=/ ) {
			s/.*=\W*(\w+)\W*/$1/;
			$workgroup = $_ if ! defined $workgroup;
		    }
		}
		close $input; $_=$save;
	    }
            s/\$SMBSERVER\%/$smbserver/g;
        }
        if( /\%WORKGROUP\%/ ) {
	    if( ! defined $workgroup || $workgroup eq "" ) {
		$workgroup = "schoollan2" ;
		my $input; my $save=$_;
                open $input, "</etc/samba/smb.conf"  ;
		while(<$input>) { chop;
		    if( /^\s\s*workgroup\s*=/ ) {
			s/.*=\W*(\w+)\W*/$1/; $workgroup = $_; last ;
		    }
		    if( /^\s+netbios\s+name\s*=/ ) {
			s/.*=\W*(\w+)\W*/$1/;
			$smbserver = $_ if ! defined $smbserver;
		    }
		}
		close $input; $_=$save;
	    }
            s/\%WORKGROUP\%/$workgroup/g;
        }
	return $_ ;
}

@WinRegKeys=( );
# read the key[=value] pairs (=value is optional) from definition file
sub fill_WinRegKeys {
    my ($file) = @_;
    my $input ;
    if ( ! open $input, "<$file" ) {
	error "Cannot open for reading Windows Register Keys file $file";
	exit 0;
     }
     while (<$input>){ chop;
	s/[^\\]#.*//; next if /^\s*$/;
	# delete leading and trailing blanks
	s/^\s*(.*?)\s*$/$1/;
	s/=ON$/=$ON/; s/=OFF$/=$OFF/; s/=$//;
	push @WinRegKeys, "$_" ;
     }
     close $input;
}

# make the .reg file to avoid the nasty windows question:
# you have not logged on at this computer before. Would you like this computer......
# should be called after the create_logon_script
sub create_Reg {
        my ($RegFileName, $arr, $hive_type, $enType) = @_;
	my $RememberHive = "\r\n";
	my $RememberWin = "\r\n" ; my $printed = 0;
	# enable and value those needed to
	foreach my $elem (@{$arr}) {
	    my $key = $elem ; $key =~ s/\s*(\S+.*\S)\s*=.*$/$1/ ;
	    my $hivepart = ""; $_ = $key; $key =~ s/.*\\//g;
	    if( /\\/ ) { s/\\[^\\]*$//; $hivepart = $_ ; }
	    if ( $key eq "" ) {
	        error "Warning: error in key definition \"$elem\": empty key"; next;
	    }
	    $_ = $elem ; 
	    if ( ! /=/ ) { $_ = "" } else {
		s/.*=\s*$//; s/^.*=\s*(.*?)\s*$/$1/; s/^ON$/$ON/; s/^OFF$/$OFF/;
	        if ( ! /^hex:[\d,A-Fa-f]*$/ && ! /^dword:\d+$/ && ! /^-$/ ) {
	            s/^"//; s/([^\\])"$/$1/; s/([^\\])"/$1\\"/g; s/^/"/; s/$/"/;
	        }
	    }
	    my $value = $_ , $found=0, $lasthive="\r\n";
	    for my $i ( 0  .. $#WindowsReg) {
		if ( defined $WindowsReg[$i]{hive} ) { $lasthive = $WindowsReg[$i]{hive} ; }
		my $hive_probe = $lasthive ; $hive_probe =~ s/\\.*//g ;
		my $match = 0;
		if ( $hive_type eq "" ) { $hive_probe = "" ; }
		if ( $hive_type eq "*" ) { $hive_probe = "*" ; }
		if ( $hive_probe ne $hive_type ) { next ; }
		if ( $hivepart ne "" ) {
		    $_ = $lasthive ; next if ! /$hivepart$/ ;
		}
		if ( $WindowsReg[$i]{key} eq $key ) {
		    $found++;
		    my $type1 = $WindowsReg[$i]{value}; $type1 =~ s/(.).*/$1/;
		    my $type2 = $value eq "" ? $type1 : $value; $type2 =~ s/(.).*/$1/;
		    if ( $type1 ne $type2 ) {
			error "Warning: key \"$key\" with value \"$value\" is not of correct type (omitted)";
			next;
		    }
		    $WindowsReg[$i]{enabled} = $enType;
                    $_ = $elem;
		    if ( $value ne "" ) { $WindowsReg[$i]{value} = $value if /=/ ; }
		}
	    }
	    error "Warning: key \"$key\" with value \"$value\" not found in database" if $found < 1; 
	} 
	for $i ( 0  .. $#WindowsReg) {
                if ( defined $WindowsReg[$i]{win} ) {
	            if ( $RememberWin ne $WindowsReg[$i]{win} ) {
			$RememberWin = $WindowsReg[$i]{win} ;
		    }
		}
		if ( defined $WindowsReg[$i]{hive} ) {
		    $printed = 0 if ( $RememberHive ne $WindowsReg[$i]{hive} );
		    $RememberHive = $WindowsReg[$i]{hive} ;
		}
		$_ = $RememberWin ; next if ! /$arch/ ;
		my $hive_probe = $RememberHive ; $hive_probe =~ s/\\.*//g ;
                if ( $hive_type eq "" ) { $hive_probe = "" ; }
                if ( $hive_type eq "*" ) { $hive_probe = "*" ; }
                if ( $hive_probe ne $hive_type ) { next ; }
		if ( ! defined $WindowsReg[$i]{enabled} &&
			$enType ne "" ) { next ; }
		if ( $enType ne $WindowsReg[$i]{enabled} && $enType ne "*" && $enType ne "" ) { next ; }
		if ( ! defined $WinRegOpened ) {
                    open HOMEREG, ">$RegFileName" ;
		    print HOMEREG "REGEDIT4\r\n";
		    $WinRegOpened = 2;
		}
		if( $WinRegOpened < 2 ) { open HOMEREG, ">>$RegFileName" ; $WinRegOpened = 2;}
		print HOMEREG "\r\n\[" . substitute($RememberHive) . "\]\r\n" if ! $printed++ ;
		if ( defined $WindowsReg[$i]{key} && defined $WindowsReg[$i]{value} ) {
		    print HOMEREG "\"" . substitute($WindowsReg[$i]{key}) . "\"=" ;
		    $_ = substitute($WindowsReg[$i]{value});
		    if ( ! /^hex.*/ && ! /^dword/ && !  /^".*"$/ ) { $_ = "\"".$_."\""; }
		    print HOMEREG $_ . "\r\n" ;
		}
	}
	return 0 if ! defined $WinRegOpened ;
	print HOMEREG "\r\n";
	close HOMEREG; $WinRegOpened = 1;
	return 1;
}

sub DumpReg {
    my $Win="Win95|Win2K|WinNT|WinXP", $i;

    print "REGEDIT4\n";
    for $i (0 .. $#WindowsReg) {
        if ( defined $WindowsReg[$i]{win} ) {
	    $Win=$WindowsReg[$i]{win};
	}
    	if ( defined $WindowsReg[$i]{hive} ) {
   	    print "\n[$WindowsReg[$i]{hive}]\n";
    	}
    	if ( defined $WindowsReg[$i]{key} ) {
    	    print "<$Win:>\t\"$WindowsReg[$i]{key}\"=";
    	    if( defined $WindowsReg[$i]{value} ) {
    		print "$WindowsReg[$i]{value}" ;
    	    }
    	    print "\n";
    	}
    }
    exit 0;
}

##################### END of REGISTRY HANDLING ROUTINE


# check if the user belongs to the group "second arg"
sub is_member {
        ($me,$grp) = @_; setgrent ;
        my $name; my $passwd; my $gid; my $members; my @names; my $person;
        while ( ($name, $passwd, $gid, $members) = getgrent ) {
                if ( $name eq $grp) {
                        @names = split / /, $members ;
                        foreach $person (@names) {
                                if( $person eq $me ){ return 1; }
                        }
                }
        }
        return 0;
}

# create a logon script to prepair various user dependent things
# usualy the attachment of needed Windows server services
sub create_logon_script {
	my $persist = "/PERSISTENT:NO" ;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
        my $datestrg = sprintf("%02.2d/%02.2d/%04.4d %02.2dh%02.2d", $mday,$mon,$year+1900,$hour,$min);
	print LOGON "\@echo off\r\n" if ( $verbose ne "" );
	print LOGON "rem -- made for $client with arch $arch on date/time $datestrg\r\n";

        if ( $arch eq "Win95" ) {
	    # removes all existing profiles on the computer a user logs in
	    # somehow this does not seem to work
	    print LOGON "rem DELTREE /y C:\\WINDOWS\\PROFILES\\*.*$verbose\r\n";
	    # makes a directory in local profiles with the name of the user
	    print LOGON "rem MD C:\\WINDOWS\\PROFILES\\$user$verbose\r\n";
	    # register the user to avoid .... see above makeing the .reg file
	    # fetch the right time from the server
	    print LOGON "NET TIME \\\\$smbserver /SET /YES$verbose\r\n";
        }

	if( $arch eq "WfWg" or $arch eq "Win95" ) { $persist = "" ; }
	# H: should be the home resource of the user
	print LOGON "NET USE H: \\\\$smbserver\\$user $persist /YES$verbose\r\n";
        # when dos programs disappear G: and P: can go away
	# where all the programs can be found on the server
	print LOGON "NET USE G: \\\\$smbserver\\pgroups\$ $persist /YES$verbose\r\n";
	# P: should be all the shared programs
	# as old programs need a "hard" network drive we keep this
	# and allow it to stay visible
	print LOGON "NET USE P: \\\\$smbserver\\programs\$ $persist /YES$verbose\r\n";
	# I: should be installments
	# next should only be there for staff (to do)
	if( is_member($user,"team") ) {
		print LOGON "NET USE I: \\\\$smbserver\\install\$ $persist /YES$verbose\r\n";
	}
	# G: has all the private programs mostly a short cut to P; residing programs
	if( $user eq "wadmin"  or is_member($user,"wadmin") ) {
		print LOGON "NET USE W: \\\\$ARGV[2]\\website $persist /YES$verbose\r\n";
	}
}

#################################################

if( $#ARGV < 0 ) { usage ; }

# the arguments we pick up
while( $_ = $ARGV[0], /^-/ ) {
	shift; last if /^--/;

	if (/^-+[Hh][Ee][Ll][Pp]$/) { usage ; }
	if (/^-[Vv]erbose/)         { $verbose = "" ; }
	elsif (/^-[Dd]ebug/)         { $verbose = "" ; }
	elsif (/^-[VvDd](.*)/)       { $verbose = "" ; }

	if (/^-[Aa]rch=(.*)/)     { $arch = $1; }
	   elsif (/^-[Aa](.*)/)       { $arch = $1; }
	if (/^-[Bb]iosname=(.*)/) { $client = $1; }
	   elsif (/^-[Bb](.*)/)       { $client = $1; }
	if (/^-[Cc]lient=(.*)/)   { $dnsclient = $1; }
	   elsif (/^-[Cc](.*)/)       { $dnsclient = $1; }
	if (/^-[Hh]ome=(.*)/)     { $home = $1; }
	   elsif (/^-[Hh]ive.*=(.*)/) { push @HiveClass, $1; }
	   elsif (/^-[Hh][Kk][Ee][yY][=_](.*)/) { push @HiveClass, "HKEY_".$1; }
	   elsif (/^-[Hh](.*)/)       { $home = $1; }
	if (/^-[Ii][Pp]=(.*)/)    { $ipclient = $1; }
	   elsif (/^-[Ii](.*)/)       { $ipclient = $1; }
	if (/^-[Nn]etlogon=(.*)/) { $netlogon = $1; }
	   elsif (/^-[Nn](.*)/)       { $netlogon = $1; }
	if (/^-+[Pp][rR][Ii][Nn][Tt]$/) { DumpReg ; }
	   elsif (/^-[Pp]rotocol=(.*)/) { $protocol = $1; }
	   elsif (/^-[Pp](.*)/)            { $protocol = $1; }
	if (/^-[Ss]erver=(.*)/)   { $smbserver = $1; }
	   elsif (/^-[Ss](.*)/)       { $smbserver = $1; }
	if (/^-[Uu]ser=(.*)/)     { $user = $1; }
	   elsif (/^-[Uu](.*)/)       { $user = $1; }

	# if keyfile is defined this program generates only a reg file
	if (/^-[Ff](.*)/) { $keyfile = $1 ; }
	if (/^-[Oo](.*)/) { $OutputRegFile = $1 ; }
	if (/^-[Tt]ype=(.*)/)	{ $EnableType = $1; }
	    elsif (/^[Tt](.*)/) { $EnableType = $1; }
}
# argument list of the command
# user dnsclient server home_dir netlogon_path arch [protocol ip]
# next is an ordered list
$i=0;
if ( ! defined $user      && $i <= $#ARGV ){ $user      = $ARGV[$i++] ;}
if ( ! defined $client    && $i <= $#ARGV ){ $client    = $ARGV[$i++] ;}
if ( ! defined $smbserver && $i <= $#ARGV ){ $smbserver = $ARGV[$i++] ;}
if ( ! defined $home      && $i <= $#ARGV ){ $home      = $ARGV[$i++] ;}
if ( ! defined $netlogon  && $i <= $#ARGV ){ $netlogon  = $ARGV[$i++] ;}
if ( ! defined $arch      && $i <= $#ARGV ){ $arch      = $ARGV[$i++] ;}
if ( ! defined $protocol  && $i <= $#ARGV ){ $protocol  = $ARGV[$i++] ;}
if ( ! defined $ipclient  && $i <= $#ARGV ){ $ipclient  = $ARGV[$i++] ;}
if ( ! defined @HiveClass ) { push @HiveClass, "HKEY_LOCAL_MACHINE" ; }
foreach $i (@HiveClass) {
	$i =~ s/^HKLM$/HKEY_LOCAL_MACHINE/;
	$i =~ s/^HKCU$/HKEY_CURRENT_USER/;
	$i =~ s/^HKCR$/HKEY_CLASSES_ROOT/;
	$i =~ s/^HKCC$/HKEY_CURRENT_CONFIG/;
	$i =~ s/^HKU$/HKEY_USERS/;
}

sub myname {
	if ( defined $dnsclient && $dnsclient ne "" ) {
		my @tmp = split('\.',$dnsclient);
		return $tmp[0];
	}
	else { return "UNKNOWN"; }
}

if ( ! defined $arch || $arch eq "" ) {
	error "Cannot run without Win?? architectured defined!";
	exit 0;
}

if ( ! defined $client || $client eq "" ) {
    $client = myname;
}
$client =~ s/\..*//g;

if ( defined $netlogon && defined $home && defined $user ) {
    # we log via syslog make sure local3.info is collected separately via /etc/syslog.conf
    system "/usr/bin/logger -t SMBclient -p local3.info \"<$user> NETLOGON client <$client> (arch: <$arch> protocol: <$protocol>)\"";

    open LOGON, ">$home/$netlogon/$user.bat";
    create_logon_script;

    # create a Windows Register file for settings we need
    #@WinRegKeys=("ProfileImagePath", "Comment","DontShowLastUser=OFF","0003\\IPAddress=%CLIENT%");
    # next is only for WinNT and higher
    @WinRegKeys=("DeleteRoamingCache=ON");
    if( defined $keyfile ) { fill_WinRegKeys ( $keyfile ); }
    # next handles the client machine Windows registry file
    # to be added the {NT}USER.{MAN,DAT,POL} Windows Regostry file setting via regedit
    # a Linux program from regutils http://www.cs.mon.ca/~michael/regutils
    # or via Samba >= V3.0 regedit/editreg and other routines

    # to do: extend this to allow this to be called for every Win Registry we have at hand
    # we know this works for Win95 USER.DAT so disable this for other archs for a while
    my $XPonce = 0;
    foreach my $HiveElement (@HiveClass) {
        if ( $HiveElement eq "HKEY_USERS" && $arch ne "Win95" ) { $HiveElement = "*" ; }
        $Output = "$home/$netlogon/$client.$HiveElement.reg" ;
        if( defined $OutputRegFile ) { $Output = $OutputRegFile; }
	# remain downwards compatable
	if ( ! defined $EnableType ) { $EnableType="init" ; }
        # to be done: HiveClass should be an array...
        if( create_Reg ( $Output,\@WinRegKeys, $HiveElement, $EnableType ) ) {
	    if ( -s $Output ) {
	        chown $user, $Output;
	        if ( $arch eq "Win95" && $HiveElement eq "HKEY_USERS" && -x "/usr/local/bin/regedit" ) {
	            chmod 0666, "$home/$netlogon/USER.DAT" ;
	            if (system "/usr/local/bin/regedit -f $home/$netlogon/USER.DAT -i $Output -q >/dev/null 2>&1" == 0 ) {
		        print LOGON "rem USER.DAT file changed via server regedit call\r\n" ;
		    } else {
		        system "/usr/bin/logger -t SMBclient -p local3.warning \"could not change USER.DAT for $user on $client\"" ;
		    }
	            chmod 0644, "$home/$netlogon/USER.DAT" ;
	        }
		# still something to do and to test here ...
	        elsif ( $arch ne "WinXP" ) {
	            print LOGON $arch ne "Win2K" ? "REGEDIT" : "REGEDT32"  if ! defined $OutputRegFile;
                    print LOGON " -s \\\\$smbserver\\Profiles\\$client.$HiveElement.reg$verbose\r\n" if ! defined $OutputRegFile;
		} else {
		    if ( -e "/etc/samba/xpreg.bat" ) {
                        system "/bin/cp /etc/samba/xpreg.bat $home/$netlogon/xpreg.bat > /dev/null 2>&1" ;
                        system "/bin/chown $user $home/$netlogon/xpreg.bat > /dev/null 2>&1" ;
                        print LOGON "IF EXIST Z:\\WinXP\\xpreg.bat CALL Z:\\WinXP\\xpreg.bat\r\n";
		    } else {
			print LOGON "rem for each key and value do it. This needs to be extended.\r\n" ;
			print LOGON "reg add \"HKCU\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\" /v \"Personal\" /t REG_SZ /d \"H:\\Mijn Documenten\" /f \\\n" ;
			print LOGON "reg add \"HKCU\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders\" /v \"Personal\" /t REG_EXPAND_SZ /d: \"%%LOGONSERVER%%\\%%USERNAME%%\\Mijn Documenten\" /f \r\n";
		    }
	        }
	    }
	}
    }

    # close logon script
    if ( $verbose eq "" ) { print LOGON "pause\r\n" ; }
    close LOGON;
    chown $user, "--silent $home/$netlogon/$user.bat $home/$netlogon/$client.reg";
}
else {
    # no makelogonscript use part
    # only for Windows registry settings generation
    if ( ! defined $OutputRegFile ) { $OutputRegFile = "/dev/stdout"; }
    if ( ! defined $EnableType ) { $EnableType="*" ; }
    if ( ! defined $keyfile ) { $keyfile = "/dev/stdin"; }
    fill_WinRegKeys ( $keyfile );
    foreach my $HiveElem (@HiveClass) {
        create_Reg( $OutputRegFile, \@WinRegKeys, $HiveElem, $EnableType);
    }
}

